@@ -26,7 +26,6 @@
 #endif
 
 #include <string.h>
-#include <math.h>
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -41,7 +40,6 @@
 
 
 
-#define GUESS_BUFFER_SIZE           512
 #define BORING_IMAGE_VARIANCE       256.0    /* tweak this if necessary */
 #define TUMBLER_GST_PLAY_FLAG_VIDEO (1 << 0) /* from GstPlayFlags */
 #define TUMBLER_GST_PLAY_FLAG_AUDIO (1 << 1) /* from GstPlayFlags */
@@ -517,42 +515,6 @@ gst_thumbnailer_play_init (TumblerFileInfo *info)
 
 
 
-static GdkPixbuf *
-gst_thumbnailer_scale_pixbuf (GdkPixbuf *source,
-                              gint       dest_width,
-                              gint       dest_height)
-{
-  gdouble wratio;
-  gdouble hratio;
-  gint    source_width;
-  gint    source_height;
-
-  /* determine source pixbuf dimensions */
-  source_width  = gdk_pixbuf_get_width  (source);
-  source_height = gdk_pixbuf_get_height (source);
-
-  /* don't do anything if there is no need to resize */
-  if (source_width <= dest_width && source_height <= dest_height)
-    return g_object_ref (source);
-
-  /* determine which axis needs to be scaled down more */
-  wratio = (gdouble) source_width  / (gdouble) dest_width;
-  hratio = (gdouble) source_height / (gdouble) dest_height;
-
-  /* adjust the other axis */
-  if (hratio > wratio)
-    dest_width = rint (source_width / hratio);
-  else
-    dest_height = rint (source_height / wratio);
-
-  /* scale the pixbuf down to the desired size */
-  return gdk_pixbuf_scale_simple (source, MAX (dest_width, 1),
-                                  MAX (dest_height, 1),
-                                  GDK_INTERP_BILINEAR);
-}
-
-
-
 static void
 gst_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
                         GCancellable               *cancellable,
@@ -568,62 +530,29 @@ gst_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   TumblerThumbnailFlavor *flavor;
   GdkPixbuf              *scaled;
   const gchar            *uri;
-  GFile                  *file;
-  GFileInputStream       *stream;
-  guchar                  buffer[GUESS_BUFFER_SIZE];
-  gssize                  size;
-  gchar                  *guessed_type, *claimed_type;
-  gboolean                uncertain, equals;
 
   /* check for early cancellation */
   if (g_cancellable_is_cancelled (cancellable))
     return;
 
-  /* Check if is a sparse video file */
   uri = tumbler_file_info_get_uri (info);
+  g_debug ("Handling URI '%s'", uri);
+
+  /* Check if is a sparse video file */
   if (tumbler_util_guess_is_sparse (info))
   {
-    g_debug ("Video file '%s' is probably sparse, skipping\n", uri);
+    g_debug ("Video file '%s' is probably sparse, skipping", uri);
+
+    /* there was an error, emit error signal */
+    g_set_error (&error, TUMBLER_ERROR, TUMBLER_ERROR_NO_CONTENT,
+                 TUMBLER_ERROR_MESSAGE_CREATION_FAILED);
+    g_signal_emit_by_name (thumbnailer, "error", uri,
+                           error->domain, error->code, error->message);
+    g_error_free (error);
+
     return;
   }
 
-  /* check if real mime type matches advertised mime type to avoid server-side request
-   * forgery: see https://gitlab.xfce.org/xfce/tumbler/-/issues/65 */
-  file = g_file_new_for_uri (uri);
-  stream = g_file_read (file, cancellable, &error);
-  g_object_unref (file);
-  if (stream == NULL)
-    {
-      g_signal_emit_by_name (thumbnailer, "error", uri, error->code, error->message);
-      g_error_free (error);
-
-      return;
-    }
-
-  size = g_input_stream_read (G_INPUT_STREAM (stream), buffer, GUESS_BUFFER_SIZE,
-                              cancellable, &error);
-  g_object_unref (stream);
-  if (size == -1)
-    {
-      g_signal_emit_by_name (thumbnailer, "error", uri, error->code, error->message);
-      g_error_free (error);
-
-      return;
-    }
-
-  guessed_type = g_content_type_guess (NULL, buffer, size, &uncertain);
-  claimed_type = g_content_type_from_mime_type (tumbler_file_info_get_mime_type (info));
-  equals = guessed_type != NULL && claimed_type != NULL
-           && g_content_type_equals (guessed_type, claimed_type);
-  g_free (guessed_type);
-  g_free (claimed_type);
-  if (uncertain || ! equals)
-    {
-      g_signal_emit_by_name (thumbnailer, "error", uri, TUMBLER_ERROR_INVALID_FORMAT,
-                             _("Thumbnail could not be inferred from file contents"));
-      return;
-    }
-
   /* get size of dest thumb */
   thumbnail = tumbler_file_info_get_thumbnail (info);
   flavor = tumbler_thumbnail_get_flavor (thumbnail);
@@ -659,7 +588,7 @@ gst_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
   if (G_LIKELY (pixbuf != NULL))
     {
       /* scale to correct size if required */
-      scaled = gst_thumbnailer_scale_pixbuf (pixbuf, width, height);
+      scaled = tumbler_util_scale_pixbuf (pixbuf, width, height);
       g_object_unref (pixbuf);
       pixbuf = scaled;
 
@@ -679,17 +608,27 @@ gst_thumbnailer_create (TumblerAbstractThumbnailer *thumbnailer,
 
       if (error != NULL)
         {
-          g_signal_emit_by_name (thumbnailer, "error",
-                                 tumbler_file_info_get_uri (info),
-                                 error->code, error->message);
+          g_signal_emit_by_name (thumbnailer, "error", uri,
+                                 error->domain, error->code, error->message);
           g_error_free (error);
         }
       else
         {
-          g_signal_emit_by_name (thumbnailer, "ready",
-                                 tumbler_file_info_get_uri (info));
+          g_signal_emit_by_name (thumbnailer, "ready", uri);
         }
     }
+  else
+    {
+      /* there was an error, emit error signal */
+      if (! g_cancellable_set_error_if_cancelled (cancellable, &error))
+        g_set_error (&error, TUMBLER_ERROR, TUMBLER_ERROR_NO_CONTENT,
+                     TUMBLER_ERROR_MESSAGE_CREATION_FAILED);
+
+      g_signal_emit_by_name (thumbnailer, "error", uri,
+                             error->domain, error->code, error->message);
+      g_error_free (error);
+    }
+
 
   g_object_unref (thumbnail);
 }
